#!/usr/bin/env python
#############################################################################
# Copyright (c) 2022 One Identity
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from axosyslog_light.driver_io.network.network_io import NetworkIO
from axosyslog_light.syslog_ng_config.statements.destinations.destination_driver import DestinationDriver
from axosyslog_light.syslog_ng_ctl.legacy_stats_handler import LegacyStatsHandler
from axosyslog_light.syslog_ng_ctl.prometheus_stats_handler import PrometheusStatsHandler


def map_transport(transport):
    mapping = {
        "tcp": NetworkIO.Transport.TCP,
        "udp": NetworkIO.Transport.UDP,
        "tls": NetworkIO.Transport.TLS,
    }
    transport = transport.replace("_", "-").replace("'", "").replace('"', "").lower()

    return mapping[transport]


def create_io(ip, options):
    transport = options["transport"] if "transport" in options else "tcp"

    return NetworkIO(ip, options["port"], map_transport(transport), False)


class NetworkDestination(DestinationDriver):
    def __init__(
        self,
        stats_handler: LegacyStatsHandler,
        prometheus_stats_handler: PrometheusStatsHandler,
        ip: str,
        **options,
    ) -> None:
        self.driver_name = "network"
        self.ip = ip
        self.io = create_io(self.ip, options)
        super(NetworkDestination, self).__init__(stats_handler, prometheus_stats_handler, [self.ip], options)

    def start_listener(self):
        self.io.start_listener()

    def stop_listener(self):
        self.io.stop_listener()

    def read_log(self):
        return self.read_logs(1)[0]

    def read_logs(self, counter):
        return self.io.read_number_of_messages(counter)

    def read_until_logs(self, logs):
        return self.io.read_until_messages(logs)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Andras Mitzki <andras.mitzki@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from axosyslog_light.driver_io.clickhouse.clickhouse_io import ClickhouseClient
from axosyslog_light.syslog_ng_config.__init__ import destringify
from axosyslog_light.syslog_ng_config.statements.destinations.destination_driver import DestinationDriver
from axosyslog_light.syslog_ng_ctl.legacy_stats_handler import LegacyStatsHandler
from axosyslog_light.syslog_ng_ctl.prometheus_stats_handler import PrometheusStatsHandler


class ClickhouseDestination(DestinationDriver):
    def __init__(
        self,
        stats_handler: LegacyStatsHandler,
        prometheus_stats_handler: PrometheusStatsHandler,
        **options,
    ) -> None:
        self.driver_name = "clickhouse"
        self.options = options
        super(ClickhouseDestination, self).__init__(stats_handler, prometheus_stats_handler, None, options)

    def read_log(self) -> dict:
        return self.read_logs()[0]

    def read_logs(self) -> list[dict]:
        return self.clickhouse_client.select_all_from_table(table_name=self.options["table"])

    def create_table(self, table_name: str, table_columns_and_types: list[tuple[str, str]] = None) -> None:
        self.clickhouse_client.create_table(table_name, table_columns_and_types)

    def delete_table(self) -> None:
        self.clickhouse_client.delete_table()

    def create_clickhouse_client(self, http_port: int) -> None:
        if "user" in self.options and "password" in self.options:
            self.clickhouse_client = ClickhouseClient(username=self.options["user"], password=destringify(self.options["password"]), http_port=http_port)



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################


class InnerSource(object):
    def __init__(self, statements=None):
        self.__group_type = "inner_source"

        if statements is None:
            self.__statements = []
        else:
            self.__statements = statements

    @property
    def group_type(self):
        return self.__group_type

    @property
    def statements(self):
        return self.__statements



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Andras Mitzki <andras.mitzki@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import logging

import psutil
from axosyslog_light.common.blocking import wait_until_true
from axosyslog_light.common.blocking import wait_until_true_custom
from axosyslog_light.common.file import copy_shared_file
from axosyslog_light.executors.process_executor import ProcessExecutor

logger = logging.getLogger(__name__)


class ClickhouseServerExecutor():
    def __init__(self, testcase_parameters) -> None:
        self.process = None
        self.clickhouse_server_ports = []
        copy_shared_file(testcase_parameters, "clickhouse_server_config.xml")
        copy_shared_file(testcase_parameters, "clickhouse_users.xml")

    def start(self, clickhouse_ports) -> None:
        command = [
            "clickhouse-server", "start",
            "--config-file", "clickhouse_server_config.xml",
        ]

        self.clickhouse_server_ports.append(clickhouse_ports.http_port)
        self.clickhouse_server_ports.append(clickhouse_ports.grpc_port)

        with open("clickhouse_server_config.xml", "r") as file:
            config = file.read()
        config = config.replace("ALLOCATED_HTTP_PORT", str(clickhouse_ports.http_port))
        config = config.replace("ALLOCATED_INTERSERVER_GRPC_PORT", str(clickhouse_ports.grpc_port))
        with open("clickhouse_server_config.xml", "w") as file:
            file.write(config)

        self.process = ProcessExecutor().start(command, "clickhouse_server.stdout", "clickhouse_server.stderr")
        if wait_until_true_custom(lambda: psutil.Process(self.process.pid).children(), timeout=0.5):
            logger.info("Clickhouse server started with child process.")
            self.process = psutil.Process(self.process.pid).children()[0]
        self.wait_for_start()
        logger.info(f"Clickhouse server started with PID: {self.process.pid}")

    def stop(self) -> None:
        self.process.terminate()
        self.wait_for_stop()
        logger.info(f"Clickhouse server with PID {self.process.pid} terminated.")

    def get_open_ports(self) -> list[int]:
        if not self.process:
            raise RuntimeError("Clickhouse server process is not running.")

        connections = psutil.Process(self.process.pid).net_connections(kind='inet')
        open_ports = sorted(set([conn.laddr.port for conn in connections if conn.laddr]))
        return open_ports

    def wait_for_start(self) -> bool:
        assert wait_until_true(lambda: self.get_open_ports() == self.clickhouse_server_ports), "Required ports are not open."

    def wait_for_stop(self) -> bool:
        assert wait_until_true(lambda: self.get_open_ports() == []), "Required ports are not closed."



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Andras Mitzki <andras.mitzki@axoflow.com>
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
import json
import logging

import clickhouse_connect

logger = logging.getLogger(__name__)


class ClickhouseClient():
    def __init__(self, username: str, password: str, http_port: int) -> None:
        self.username = username
        self.password = password
        self.http_port = http_port
        self.table_name = None

    def create_client(self):
        return clickhouse_connect.get_client(host="localhost", username=self.username, password=self.password, port=self.http_port)

    def create_table(self, table_name: str, table_columns_and_types: list[tuple[str, str]] = None) -> None:
        def construct_create_table_query(table_name: str, table_columns_and_types: list[tuple[str, str]]) -> str:
            self.table_name = table_name
            columns_definition = ", ".join([f'"{col}" {typ}' for col, typ in table_columns_and_types])
            prim_key_col_name = table_columns_and_types[0][0]
            create_table_query = f"CREATE TABLE {self.table_name} ({columns_definition}) ENGINE MergeTree() PRIMARY KEY (%s);" % prim_key_col_name
            return create_table_query

        create_table_query = construct_create_table_query(table_name, table_columns_and_types)

        client = self.create_client()
        client.command(create_table_query)
        client.close()

        logger.info(f"Table created: {table_name} with {create_table_query}")

    def delete_table(self) -> None:
        if self.table_name:
            client = self.create_client()
            client.command(f'DROP TABLE IF EXISTS {self.table_name};')
            client.close()
            logger.info(f"Table deleted: {self.table_name}")

    def select_all_from_table(self, table_name: str) -> list:
        if not table_name:
            raise ValueError("Table name must be provided to select data.")
        client = self.create_client()
        query = f"SELECT * FROM {table_name} FORMAT JSONEachRow;"
        logger.info(f"Executing Clickhouse query: {query}")
        try:
            query_res = client.command(query)
            rows = [json.loads(line) for line in query_res.splitlines()]
            logger.info(f"Query returned {len(rows)} rows")
            return rows
        except Exception as e:
            logger.error(f"Error executing query: {e}")
            return []
        finally:
            client.close()



#!/usr/bin/env python
#############################################################################
# Copyright (c) 2025 Axoflow
# Copyright (c) 2025 Attila Szakacs <attila.szakacs@axoflow.com>
# Copyright (c) 2020 One Identity
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# As an additional exemption you are allowed to compile & link against the
# OpenSSL libraries as published by the OpenSSL project. See the file
# COPYING for details.
#
#############################################################################
from __future__ import annotations

import os
import typing
from pathlib import Path
from subprocess import DEVNULL
from subprocess import Popen as subprocess_Popen

from axosyslog_light.executors.process_executor import ProcessExecutor
from axosyslog_light.helpers.loggen.loggen_executor import LoggenExecutor
from axosyslog_light.helpers.loggen.loggen_executor import LoggenStartParams
from psutil import Popen


class LoggenDockerExecutor(LoggenExecutor):
    def __init__(self, image_name: str) -> None:
        self.__image_name = image_name
        super().__init__()

    def _start(self, start_params: LoggenStartParams, stderr: Path, stdout: Path, instance_index: int) -> Popen:
        subprocess_Popen(["docker", "rm", "-f", f"loggen_{instance_index}"], stdout=DEVNULL, stderr=DEVNULL).wait()

        paths: typing.Set[Path] = {
            Path.cwd().absolute(),
        }
        if start_params.read_file:
            paths.add(Path(start_params.read_file).parent.absolute())

        command = ["docker", "run", "--rm", "-i"]
        command += ["--entrypoint", "loggen"]
        command += ["--name", f"loggen_{instance_index}"]
        command += ["--workdir", str(Path.cwd().absolute())]
        command += ["--user", f"{os.getuid()}:{os.getgid()}"]
        command += ["-e", f"PUID={os.getuid()}", "-e", f"PGID={os.getgid()}"]
        command += ["--network", "host"]

        for path in paths:
            command += ["-v", f"{path}:{path}"]

        command += [self.__image_name]
        command += start_params.format()

        return ProcessExecutor().start(command, stdout, stderr)

    def _copy(self) -> LoggenExecutor:
        return LoggenDockerExecutor(self.__image_name)
